package tech.riemann.ims.controller.platform.material;

import club.zhcs.lina.starter.exception.BizException;
import club.zhcs.lina.utils.enums.Codebook;
import club.zhcs.lina.utils.enums.ICodeBook;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.nutz.lang.Strings;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import tech.riemann.ims.dto.response.TypeTree;
import tech.riemann.ims.entity.material.Type;
import tech.riemann.ims.service.material.ITypeService;

import java.util.Arrays;
import java.util.List;

/**
 * @author mayong
 * @since 2025/3/2 14:14
 */
@RestController
@Slf4j
@Tag(name = "Type", description = "物料类型管理")
@RequiredArgsConstructor
public class TypeController {

    private final ITypeService typeService;

    @GetMapping("areas")
    @Operation(summary = "查询类型")
    public List<Type> areas(
            @Parameter(description = "搜索关键词") @RequestParam(required = false, defaultValue = "") String key) {
        boolean hasLike = Strings.isNotBlank(key);
        key = String.format("%%%s%%", key);
        return typeService.list(
                Wrappers.<Type>lambdaQuery()
                        .like(hasLike, Type::getName, key));
    }

    @PostMapping("area")
    @Operation(summary = "增加/更新地区")
    public Type addAndModify(@Validated @Parameter(description = "类型") @RequestBody Type area) {
        if (area.getId() == null) {
            typeService.save(area);
        } else {
            typeService.updateById(area);
        }
        return area;
    }

    @DeleteMapping("area/{id}")
    @Operation(summary = "删除类型")
    public void deleteArea(@Parameter(description = "id", required = true) @PathVariable long id) {
        if (!typeService.removeById(id)) {
            throw BizException.create("删除失败!");
        }
    }

    @GetMapping("area/levels")
    @Operation(summary = "级别")
    public List<Codebook> levels() {
        return Arrays.stream(Type.Level.values()).map(ICodeBook::build).toList();
    }

    @GetMapping("area/trees")
    @Operation(summary = "类型树")
    public List<TypeTree> trees() {
        List<Type> all = typeService.list();
        List<TypeTree> data = all.stream().map(area -> TypeTree.builder()
                        .value(area.getNo())
                        .label(area.getName())
                        .parentNo(area.getParentNo())
                        .build())
                .toList();
        return TypeTree.buildTree(data, "0");
    }
}
